#!/bin/bash
#
#===============================================================================
# Quita recursivamente los caracteres problemáticos de los nombres de fichero. 
# Ejecutarlo varias veces, a la primera puede que no los sustituya todos. 
# Por ejemplo si find busca el  - inicial y el nombre tiene \ find no muestra
# bien el nombre porque piensa que \ está escapando al siguiente carácter. Solo
# lo muestra bien cuando busca específicamente la \. Usa find y sed y cada uno
# interpreta los caracteres problemáticos de diferente manera por lo que tuve que 
# hacer el guión un poco empíricamente. Hay un directorio con ficheros 
# problemáticos de ejemplo. Listarlos con ls -pQR | less
# No quita el carácter de nueva línea ni el tabulador. Existe el programa detox
# para lo mismo.
#===============================================================================
#
# http://www.controlledvocabulary.com/imagedatabases/filename_limits.html
# http://www.dwheeler.com/essays/fixing-unix-linux-filenames.html
# caracteres permitidos:
# 1234567890
# abcdefghijklmnopqrstuvwxyz
# .
# _

# no puede empezar por . (porque estonces es oculto)
# no puede haber espacios, ni ñ, ni tildes.
# se pueden usar mayúsculas pero para la máxima compatibilidad entre sistemas 
# no usarlas.
# El - da problemas con ISO-9660 y en algunos servidores web.

# Longitud: 8 caracteres más el punto más la extensión sería el máximo de 
# portabilidad.
# Algo razonable actualmente sería 64 o 100 como mucho. 

# Linux acepta 255 caracteres para nombre y 4096 para ruta completa. Hasefroch 
# creo que acepta 255 para nombre incluyendo la ruta completa.


# 1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890
# aaaaaaaa10bbbbbbbb20cccccccc30dddddddd40eeeeeeee50ffffffff60gggggggg70hhhhhhhh80iiiiiiii90jjjjjjj100
# 20140631_practica_de_quimica_fisica_nivel_1-ano_0_bis

# Para este código genérico de modificación de nombres de fichero:
#
#find $1 -depth -name '*ñ*' | while read fichero
#do nuevofichero=`echo "$fichero" | sed 's/ñ/n/g'`
#mv "$fichero" "$nuevofichero"
#done
#
# es necesario ejecutarlo varias veces porque find realiza una búsqueda
# inicial con los nombres originales y conforme van cambiando los nombres
# de los directorios llega un momento en que ya no se encuentran los
# ficheros puesto que sus rutas absolutas han cambiado." Se podría
# resolver usando la opción -execdir en vez de -exec y la opción
# -depth, pero no he podido poner todos los comandos en una tubería. En
# minusculas.sh sí funciona: find $1 -depth -execdir rename 'tr/A-Z/a-z/'
# "{}" \+
#
# En este guión está resuelto con la opción -depth y separando la
# ruta del nombre del fichero y luego modificando solo el nombre del
# fichero. Si solo usamos -depth no vale porque sed modifica también
# la ruta. Si solo usamos el separar ruta y nombre no funciona porque se
# modifican las rutas más cercanas a la raíz de búsqueda y luego los
# nombres de las ramas ya no coinciden con los que find encontró primero.
#

if [ $# -ne 1 ]
then
  echo "Uso: "${0##*/}" ruta"
  exit 0
fi

# 35 caracteres problemáticos los sustituimos por _. El - solo sutituimos si 
# es el primer carácter del nombre, si está en medio o final no da problemas.

# El primer carácter es el - al principio de nombre, lo sustituimos por _:

find $1 -depth -iname '-*' | while read fichero
do
  ruta="${fichero%/*}"  
  nombre="${fichero##*/}"
  sust="_"
  nuevonombre=$(echo "${nombre}" | sed   "s/-/${sust}/")

# comprobamos si ya existe algún fichero o directorio y si es así le añadimos 
# más _ al nombre:
 
  while [ -e $1/"${nuevonombre}" ]
  do   
    sust=${sust}_

    nuevonombre=$(echo "${nombre}" | sed   "s/-/${sust}/")
  done
  mv -v "${ruta}/${nombre}" "${ruta}/${nuevonombre}"
done


# los 32 siguientes son:
#                  123 45678901234567890123456789012
#                           10        20        30

# con bucle for no funciona para ! + ª así que usamos while:
find $1 -depth -iname '*[()[\]{}<>*+=!?"·$%&^ºª,;:|#~`@¡¿€]*' | while read fichero
do
  ruta="${fichero%/*}"
  nombre=${fichero##*/}

# Para que sed sustituya [ y ] hay que ponerlos juntos al principio de la 
# sustitución juntos. Fuente: http://www.grymoire.com/Unix/Regular.html#uh-6

  sust="_"

  nuevonombre=$(echo "${nombre}" | sed   "s/[][(){}<>*+=\!?\"·$%&^ºª,;:|#~\`@¡¿€]/${sust}/g")

# comprobamos si ya existe algún fichero o directorio y si es así le añadimos 
# más _ al nombre:

  while [ -e $1/"${nuevonombre}" ]
  do   
    sust=${sust}_
  
    nuevonombre=$(echo "${nombre}" | sed   "s/[][(){}<>*+=\!?\"·$%&^ºª,;:|#~\`@¡¿€]/${sust}/g")

  done
  mv -v "${ruta}/${nombre}" "${ruta}/${nuevonombre}"
done


# el 34 y el 35 los ponemos a parte porque o dan problemas en while, en find o 
# o en sed y requieren un tratamiento personalizado.

# , el 34:
# con el bucle while no funciona, tiene que ser con for:
for fichero in $(find $1 -depth -name '*\\*')
do
  ruta="${fichero%/*}"  
  nombre="${fichero##*/}"
  sust="_"
  nuevonombre=$(echo "${nombre}" | sed   "s/[\]/${sust}/g")

# comprobamos si ya existe algún fichero o directorio y si es así le añadimos 
# más _ al nombre:

  while [ -e $1/"${nuevonombre}" ]
  do   
    sust=${sust}_

    nuevonombre=$(echo "${nombre}" | sed   "s/[\]/${sust}/g")
  done
  mv -v "${ruta}/${nombre}" "${ruta}/${nuevonombre}"
done


# y el 35:
find $1 -depth -iname "*'*" | while read fichero
do
  ruta="${fichero%/*}"  
  nombre="${fichero##*/}"
  sust="_"
  nuevonombre=$(echo "${nombre}" | sed   "s/'/${sust}/g")

# comprobamos si ya existe algún fichero o directorio y si es así le añadimos 
# más _ al nombre:

  while [ -e $1/"${nuevonombre}" ]
  do   
    sust=${sust}_

    nuevonombre=$(echo "${nombre}" | sed   "s/'/${sust}/g")
  done
  mv -v "${ruta}/${nombre}" "${ruta}/${nuevonombre}"
done 
